﻿/*
 * Copyright 2011 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Text;

namespace Vajhoej.StaticWeaver
{
    /// <summary>
    /// Class MethodJoinpoint contains the information made available to advices.
    /// </summary>
    public class MethodJoinpoint
    {
        private string typnam;
        private string metnam;
        private object res;
        private object[] args;
        private object inst;
        private bool skip;
        private object ctx;
        private Exception ex;
        private object call;
        /// <summary>
        /// Create instance of MethodJoinput with the specified properties.
        /// </summary>
        /// <param name="inst">Instance object</param>
        /// <param name="typnam">Type name</param>
        /// <param name="metnam">Method name</param>
        /// <param name="args">Arguments</param>
        public MethodJoinpoint(object call, object inst, string typnam, string metnam, params object[] args)
        {
            this.typnam = typnam;
            this.metnam = metnam;
            this.res = null;
            this.args = args;
            this.inst = inst;
            this.skip = false;
            this.ctx = null;
            this.ex = null;
            this.call = call;
        }
        /// <summary>
        /// Type name.
        /// </summary>
        public string TypeName
        {
            get { return typnam; }
            set { typnam = value; }
        }
        /// <summary>
        /// Method name.
        /// </summary>
        public string MethodName
        {
            get { return metnam; }
            set { metnam = value; }
        }
        /// <summary>
        /// Result value.
        /// </summary>
        public object Result
        {
            get { return res; }
            set { res = value; }
        }
        /// <summary>
        /// Arguments.
        /// </summary>
        public object[] Arguments
        {
            get { return args; }
            set { args = value; }
        }
        /// <summary>
        /// Instance object.
        /// </summary>
        public object Instance
        {
            get { return inst; }
            set { inst = value; }
        }
        /// <summary>
        /// Skip call.
        /// </summary>
        public bool SkipCall
        {
            get { return skip; }
            set { skip = value; }
        }
        /// <summary>
        /// Context object (to be used between advices).
        /// </summary>
        public object Context
        {
            get { return ctx; }
            set { ctx = value; }
        }
        /// <summary>
        /// Caugth exception.
        /// </summary>
        public Exception CaugthException
        {
            get { return ex; }
            set { ex = value; }
        }
        /// <summary>
        /// Caller object.
        /// </summary>
        public object Caller
        {
            get { return call; }
            set { inst = call; }
        }
        /// <summary>
        /// Return signature like representation of MethodJoinpoint.
        /// </summary>
        /// <returns>Representation of MethodJoinpoint</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append(typnam);
            sb.Append(".");
            sb.Append(metnam);
            sb.Append("(");
            foreach(object arg in args)
            {
                if(sb[sb.Length - 1] != '(')
                {
                    sb.Append(",");
                }
                sb.Append(arg.ToString());
            }
            sb.Append(")");
            return sb.ToString();
        }
    }
}
