﻿/*
 * Copyright 2011 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Collections.Generic;
using System.Reflection;

using NUnit.Framework;

using Vajhoej.StaticWeaver.Implementation;

namespace Test
{
    // class TestAspect are defined in TestAttributes
    [TestFixture]
    public class TestAspectLoad
    {
        [Test]
        public void TestSearch()
        {
            Signature2Pointcut s2p = new Signature2Pointcut();
            Pointcut2Advice p2a = new Pointcut2Advice();
            AspectLoad.Search(Assembly.GetExecutingAssembly().Location, s2p, p2a, false);
            IList<string> pointcuts = s2p.Find(new Signature("* *::*(*)"));
            Assert.AreEqual(1, pointcuts.Count, "number of pointcuts");
            Assert.AreEqual("TestPointcut", pointcuts[0], "pointcut #0");
            IList<AdviceInfo> advices = p2a.Find("TestPointcut");
            Assert.AreEqual(2, advices.Count, "number of advices");
            Assert.AreEqual(AdviceType.BeforeCall, advices[0].Type, "advice #0 type");
            Assert.AreEqual("TestAdvice", advices[0].Method.Name, "advice #0 method namee");
            Assert.AreEqual(AdviceType.AfterCall, advices[1].Type, "advice #1 type");
            Assert.AreEqual("TestAdvice", advices[1].Method.Name, "advice #1 method namee");
        }
        [Test]
        public void TestFind()
        {
            Signature2Pointcut s2p = new Signature2Pointcut();
            Pointcut2Advice p2a = new Pointcut2Advice();
            AspectLoad.Search(Assembly.GetExecutingAssembly().Location, s2p, p2a, false);
            IList<AdviceInfo> advices = AspectLoad.Find(new Signature("* *::*(*)"), s2p, p2a);
            Assert.AreEqual(2, advices.Count, "number of advices");
            Assert.AreEqual(AdviceType.BeforeCall, advices[0].Type, "advice #0 type");
            Assert.AreEqual("TestAdvice", advices[0].Method.Name, "advice #0 method namee");
            Assert.AreEqual(AdviceType.AfterCall, advices[1].Type, "advice #1 type");
            Assert.AreEqual("TestAdvice", advices[1].Method.Name, "advice #1 method namee");
        }
    }
}
