﻿/*
 * Copyright 2011 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;

using NUnit.Framework;

using Vajhoej.StaticWeaver;

namespace Test
{
    [TestFixture]
    public class TestJoinpoint
    {
        [Test]
        public void TestBasic()
        {
            MethodJoinpoint mjp = new MethodJoinpoint(null, null, "NS.T", "M", 123, "ABC");
            mjp.Result = "X";
            Assert.AreEqual("NS.T", mjp.TypeName, "type name");
            Assert.AreEqual("M", mjp.MethodName, "method name");
            Assert.AreEqual(2, mjp.Arguments.Length, "number arguments");
            Assert.AreEqual(123, mjp.Arguments[0], "argument #0");
            Assert.AreEqual("ABC", mjp.Arguments[1], "argument #1");
            Assert.AreEqual("X", mjp.Result, "result");
        }
        [Test]
        public void TestAdvanced()
        {
            string o = "XXX";
            string ctx = "It works";
            MethodJoinpoint mjp = new MethodJoinpoint(o, o, "NS.T", "M", 123, "ABC");
            mjp.Result = "X";
            mjp.Context = ctx;
            Assert.AreEqual("NS.T", mjp.TypeName, "type name");
            Assert.AreEqual("M", mjp.MethodName, "method name");
            Assert.AreEqual(2, mjp.Arguments.Length, "number arguments");
            Assert.AreEqual(123, mjp.Arguments[0], "argument #0");
            Assert.AreEqual("ABC", mjp.Arguments[1], "argument #1");
            Assert.AreEqual("X", mjp.Result, "result");
            Assert.AreEqual(o, mjp.Instance, "instance");
            Assert.AreEqual(ctx, mjp.Context, "context");
            Assert.AreEqual(false, mjp.SkipCall, "skip call");
            Assert.AreEqual(o, mjp.Caller, "caller");
        }
    }
}
