﻿/*
 * Copyright 2011 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Collections.Generic;

using Vajhoej.StaticWeaver.Implementation;

namespace yaaopf
{
    /// <summary>
    /// Class Program is the main class of yaaopf command line utility.
    /// </summary>
    public class Program
    {
        private const string VERSION = "YAAOPF V1.1";
        private const string IA_SHORT = "ia";
        private const string IA_LONG = "inputassembly";
        private const string GA_SHORT = "ga";
        private const string GA_LONG = "glueassembly";
        private const string AA_SHORT = "aa";
        private const string AA_LONG = "aspectassembly";
        private const string DBG = "debug";
        private static void ShowVersion()
        {
            Console.WriteLine(VERSION);
            Console.WriteLine();
        }
        /// <summary>
        /// Main method.
        /// </summary>
        /// <param name="args">Arguments like /ia:inputassembly /ga:glueassembly /aa:aspectassembly</param>
        public static void Main(string[] args)
        {
            List<string> inputassemblies = new List<string>();
            string glueassembly = "YaaopfGlue.dll";
            List<string> aspectassemblies = new List<string>();
            bool dbg = false;
            foreach(string arg in args)
            {
                if(arg.StartsWith("/" + IA_SHORT + ":"))
                {
                    inputassemblies.Add(arg.Substring(IA_SHORT.Length + 2));
                }
                if(arg.StartsWith("/" + IA_LONG + ":"))
                {
                    inputassemblies.Add(arg.Substring(IA_LONG.Length + 2));
                }
                if(arg.StartsWith("/" + GA_SHORT + ":"))
                {
                    glueassembly = arg.Substring(GA_SHORT.Length + 2);
                }
                if(arg.StartsWith("/" + GA_LONG + ":"))
                {
                    glueassembly = arg.Substring(GA_LONG.Length + 2);
                }
                if(arg.StartsWith("/" + AA_SHORT + ":"))
                {
                    aspectassemblies.Add(arg.Substring(AA_SHORT.Length + 2));
                }
                if(arg.StartsWith("/" + AA_LONG + ":"))
                {
                    aspectassemblies.Add(arg.Substring(AA_LONG.Length + 2));
                }
                if(arg == "/" + DBG)
                {
                    dbg = true;
                }
            }
            if(inputassemblies.Count < 1 || aspectassemblies.Count < 1)
            {
                ShowVersion();
                Console.WriteLine("Usage");
                Console.WriteLine("  yaaopf /ia:<inputassembly> /ga:<glueassembly> /aa:<aspectassembly>");
                Environment.Exit(1);
            }
            else
            {
                if(dbg) ShowVersion();
                AssemblyProcessor.Weave(inputassemblies, glueassembly, aspectassemblies, dbg);
            }
        }
    }
}